// UrsOneShotBase
// 
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-timing.html
// Created: 2016-05-04
//
// Version 1.0 (2016-05-04)
// - Basis-Version
//
// Version 1.1 (2019-03-21)
// - Die Angabe der Intervalle erfolgt in ms. Umbenennung der Funktionsparameter vom "microSeconds" in "milliSeconds".
//
// Version 1.2 (2019-12-14)
// - Die Zugiffsstufe der internen Variablen von 'private' nach 'protected' gendert.
// - '_isRunning', '_isArmed' als 'volatile' deklariert.

#ifndef _URSONESHOTBASE_h
#define _URSONESHOTBASE_h

#include <Arduino.h>

extern "C" {
#include "user_interface.h" 
}

class UrsOneShotBase
{ friend void UrsOneShotBaseCallbackWrapper(void *);

protected:
  os_timer_t _osTimer;               // Interne Struktur zur Bedienung des Timers.
  volatile bool _isArmed = false;    // True: os_timer ist geladen.
  volatile bool _isRunning = false;  // True: es wird auf die Ausfhrung eines Schusses gewartet.

public:
  void start(uint32_t milliSeconds);       // Startet Timer.
  void stop();                             // Stoppt den Timer. Entldt os_timer. 
  bool isRunning() { return _isRunning; }; // True, wenn auf die Ausfhrung eines Schusses gewartet wird.

protected:
  UrsOneShotBase();
  ~UrsOneShotBase() { stop(); }   // Entldt den Timer, unterbricht das Warten.
  virtual void timerCallback() = 0;        // Funktion, die beim Ablauf des Timers aufgerufen wird.
};
#endif

