// UrsTickerBase
// 
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-timing.html
// Created: 2016-05-04
//
// Version 1.0 (2016-05-04)
// - Basis-Version
//
// Version 1.1 (2019-03-21)
// - Die Angabe der Intervalle erfolgt in ms. Umbenennung der Funktionsparameter vom "microSeconds" in "milliSeconds".
//
// Version 1.2 (2019-12-14)
// - Die Zugiffsstufe der internen Variablen von 'private' nach 'protected' gendert.
// - '_isRunning' als 'volatile' deklariert.

#ifndef _URSTICKERBASE_h
#define _URSTICKERBASE_h

#include <Arduino.h>

extern "C" {
#include "user_interface.h" 
}

class UrsTickerBase {
   friend void UrsTickerBaseCallbackWrapper(void *); // Ermglicht Aufruf von timerCallback()

protected:
   os_timer_t _osTimer;        // Interne Struktur zur Bedienung des Timers.
   uint32_t _Interval;         // Tick-Interval in ms.
   volatile bool _isRunning = false;    // True: os_timer ist geladen. 

public:
   uint32_t getInterval() { return _Interval; };  // Liefert das aktuell eingestellte Intervall in ms.
   void setInterval(uint32_t milliSeconds);       // Setzt das Intervall auf einen neuen Wert.
   void start();                                  // Startet den Ticker. Ggf. Neustart.
   void start(uint32_t milliSeconds);             // Startet den Ticker (ggf. Neustart) mit dem angegebenen Intervall
   void stop();                                   // Stoppt den Ticker
   bool isRunning() { return _isRunning; };       // True: der Ticker ist aktiv.

protected:
   UrsTickerBase(uint32_t milliSeconds);       // Initialisiert eine neue Instanz der UrsTickerBase-Klasse mit dem angegebenen Intervall.
   ~UrsTickerBase() { stop(); }                // Stoppt den Ticker.
   virtual void timerCallback() = 0;           // Funktion, die beim Ablauf des Timers aufgerufen wird.
};
#endif

